@@ -1,7 +1,6 @@
-use std::{
-    collections::{BTreeSet, HashMap},
-    iter::zip,
-};
+use std::{collections::BTreeSet, iter::zip};
+
+use crate::hashmaps::FxHashMap as HashMap;
 
 use crate::{
     opt_einsum::EinsumSpec,
@@ -10,10 +9,9 @@ use crate::{
 };
 use num_bigint::BigUint;
 
-use super::prelude::*;
 use super::{
-    apply_fn_cache, deep_map_op, deep_map_op_pre_new_children, deep_map_unwrap, visit_circuit,
-    ArrayConstant, Circuit, CircuitNode, CircuitRc, HashBytes, Index,
+    apply_fn_cache, deep_map_op_pre_new_children, deep_map_unwrap, visit_circuit, ArrayConstant,
+    Circuit, CircuitNode, CircuitRc, HashBytes, Index,
 };
 use crate::tensor_util::TorchDeviceDtype;
 
@@ -145,7 +143,7 @@ pub fn get_leaves(circuit: CircuitRc) -> Vec<CircuitRc> {
 #[pyfunction]
 pub fn get_all_einsum_specs(circuit: CircuitRc) -> Vec<EinsumSpec> {
     let mut result: Vec<EinsumSpec> = vec![];
-    visit_circuit(&**circuit, |c| {
+    visit_circuit(&circuit, |c| {
         if let Circuit::Einsum(node) = c {
             result.push(node.get_spec());
         }
@@ -161,7 +159,7 @@ pub fn toposort_circuit(circuit: CircuitRc) -> Vec<CircuitRc> {
             *num_refs.entry(child).or_insert(0) += 1;
         }
     });
-    let mut ready: BTreeSet<CircuitRc> = BTreeSet::from([circuit.clone()]);
+    let mut ready: BTreeSet<CircuitRc> = BTreeSet::from([circuit]);
     let mut result: Vec<CircuitRc> = vec![];
     while !ready.is_empty() {
         let here = ready.pop_first().unwrap();
