@@ -1,35 +1,31 @@
 use crate::hashmaps::{AHashSet as HashSet, FxHashMap as HashMap};
+use crate::py_types::{
+    einops_repeat, einsum_py, make_diagonal_py, scalar_to_tensor, scalar_to_tensor_py, PY_UTILS,
+};
+use crate::pyo3_prelude::*;
 use crate::smallvec::Sv;
 use crate::sv;
-use crate::{
-    all_imports::TorchDeviceDtypeOp,
-    new_rc_unwrap,
-    py_types::{PY_EINSUM, PY_MAKE_DIAGONAL},
-    pyo3_prelude::*,
-};
 use itertools::{zip, Itertools};
 use macro_rules_attribute::apply;
 use num_bigint::BigUint;
-use pyo3::types::IntoPyDict;
 use std::cmp::Ordering;
 
 use super::{
     algebraic_rewrite::{add_deduplicate, add_flatten_once, remove_add_few_input},
     prelude::*,
+    py_circuit_items::PY_CIRCUIT_ITEMS,
     CachedCircuitInfo, CircuitConstructionError, PyCircuitBase, ScalarConstant, Shape,
 };
 use crate::{
     circuit_node_auto_impl,
     circuit_node_extra_impl,
+    new_rc_unwrap,
     opt_einsum::EinsumSpec,
-    py_types::{
-        ExtraPySelfOps, PyEinsumAxes, Tensor, PY_CIRC_COMPILER_UTIL, PY_EINOPS_REPEAT,
-        PY_SCALAR_TO_TENSOR, PY_TORCH,
-    },
+    py_types::{ExtraPySelfOps, PyEinsumAxes, Tensor},
     rearrange_spec::RearrangeSpec, // TODO: use new rearrange spec
     tensor_util::{
         broadcast_shapes, Slice, TensorAxisIndex, TensorIndex, TensorInvariantError,
-        TorchDeviceDtype,
+        TorchDeviceDtype, TorchDeviceDtypeOp,
     },
     util::dict_to_list,
     util::{hashmap_collect_except_duplicates, EinInt},
@@ -124,38 +120,27 @@ impl CircuitNode for Einsum {
 
     fn eval_tensors(
         &self,
-        tensors: &[crate::py_types::Tensor],
+        tensors: &[Tensor],
         device_dtype: &TorchDeviceDtype,
-    ) -> Result<crate::py_types::Tensor, super::TensorEvalError> {
+    ) -> Result<Tensor, super::TensorEvalError> {
         Python::with_gil(|py| {
             if self.args.is_empty() {
-                return Ok(PY_SCALAR_TO_TENSOR
-                    .call(py, (1.0, Vec::<usize>::new(), device_dtype.clone()), None)
-                    .unwrap()
-                    .extract::<Tensor>(py)
-                    .unwrap());
+                return Ok(scalar_to_tensor_py(py, 1., sv![], device_dtype.clone())?);
             }
             let out_axes_deduped: EinsumAxes = self.out_axes.iter().unique().cloned().collect();
-            let result_non_diag: Tensor =
-                PY_EINSUM
-                    .call(
-                        py,
-                        (
-                            zip(tensors.iter().cloned(), self.input_axes().cloned())
-                                .collect::<Vec<(Tensor, EinsumAxes)>>(),
-                            out_axes_deduped.clone(),
-                        ),
-                        None,
-                    )?
-                    .extract(py)?;
+            let result_non_diag: Tensor = einsum_py(
+                py,
+                zip(tensors.iter().cloned(), self.input_axes().cloned()).collect(),
+                out_axes_deduped.clone(),
+            )?;
+
             if out_axes_deduped.len() != self.out_axes.len() {
-                Ok(PY_MAKE_DIAGONAL
-                    .call(
-                        py,
-                        (result_non_diag, out_axes_deduped, self.out_axes.clone()),
-                        None,
-                    )?
-                    .extract(py)?)
+                Ok(make_diagonal_py(
+                    py,
+                    &result_non_diag,
+                    out_axes_deduped,
+                    self.out_axes.clone(),
+                )?)
             } else {
                 Ok(result_non_diag)
             }
@@ -574,19 +559,11 @@ impl CircuitNode for Add {
 
     fn eval_tensors(
         &self,
-        tensors: &[crate::py_types::Tensor],
+        tensors: &[Tensor],
         device_dtype: &TorchDeviceDtype,
-    ) -> Result<crate::py_types::Tensor, super::TensorEvalError> {
+    ) -> Result<Tensor, super::TensorEvalError> {
         if tensors.is_empty() {
-            return Python::with_gil(|py| {
-                Ok::<crate::py_types::Tensor, super::TensorEvalError>(
-                    PY_SCALAR_TO_TENSOR
-                        .call(py, (0.0, Vec::<usize>::new(), device_dtype.clone()), None)
-                        .unwrap()
-                        .extract::<Tensor>(py)
-                        .unwrap(),
-                )
-            });
+            return Ok(scalar_to_tensor(0.0, sv!(), device_dtype.clone())?);
         }
         let mut out = tensors[0].clone();
         Python::with_gil(|py| {
@@ -741,22 +718,14 @@ impl CircuitNode for Rearrange {
 
     fn eval_tensors(
         &self,
-        tensors: &[crate::py_types::Tensor],
+        tensors: &[Tensor],
         _device_dtype: &TorchDeviceDtype,
-    ) -> Result<crate::py_types::Tensor, super::TensorEvalError> {
-        Python::with_gil(|py| {
-            PY_EINOPS_REPEAT
-                .call(
-                    py,
-                    (tensors[0].clone(), self.spec.to_einops_string()),
-                    Some(
-                        HashMap::<String, u64>::from_iter(self.spec.letter_sizes())
-                            .into_py_dict(py),
-                    ),
-                )
-                .map_err(|err| err.into())
-                .map(|x| x.extract(py).unwrap())
-        })
+    ) -> Result<Tensor, super::TensorEvalError> {
+        Ok(einops_repeat(
+            &tensors[0],
+            self.spec.to_einops_string(),
+            self.spec.letter_sizes(),
+        )?)
     }
 
     fn intermediate_cost_bound(&self) -> usize {
@@ -917,11 +886,12 @@ impl CircuitNode for Index {
 
     fn eval_tensors(
         &self,
-        tensors: &[crate::py_types::Tensor],
+        tensors: &[Tensor],
         _device_dtype: &TorchDeviceDtype,
-    ) -> Result<crate::py_types::Tensor, super::TensorEvalError> {
+    ) -> Result<Tensor, super::TensorEvalError> {
         Python::with_gil(|py| {
-            PY_CIRC_COMPILER_UTIL
+            PY_CIRCUIT_ITEMS
+                .circ_compiler_util
                 .getattr(py, "IndexUtil")
                 .unwrap()
                 .getattr(py, "apply_dimwise")
@@ -1079,11 +1049,12 @@ impl CircuitNode for Concat {
 
     fn eval_tensors(
         &self,
-        tensors: &[crate::py_types::Tensor],
+        tensors: &[Tensor],
         _device_dtype: &TorchDeviceDtype,
-    ) -> Result<crate::py_types::Tensor, super::TensorEvalError> {
+    ) -> Result<Tensor, super::TensorEvalError> {
         Python::with_gil(|py| {
-            PY_TORCH
+            PY_UTILS
+                .torch
                 .getattr(py, "cat")
                 .unwrap()
                 .call(py, (tensors.to_vec(), self.axis), None)
@@ -1250,11 +1221,12 @@ impl CircuitNode for Scatter {
 
     fn eval_tensors(
         &self,
-        tensors: &[crate::py_types::Tensor],
+        tensors: &[Tensor],
         _device_dtype: &TorchDeviceDtype,
-    ) -> Result<crate::py_types::Tensor, super::TensorEvalError> {
+    ) -> Result<Tensor, super::TensorEvalError> {
         Python::with_gil(|py| {
-            PY_CIRC_COMPILER_UTIL
+            PY_CIRCUIT_ITEMS
+                .circ_compiler_util
                 .getattr(py, "ScatterFn")
                 .unwrap()
                 .call(py, (self.index.clone(), self.info().shape.clone()), None)
