@@ -13,7 +13,7 @@ use super::{
     print::rust_expression_notation_circuit,
     ArrayConstant, Circuit, CircuitNode, CircuitRc,
 };
-use crate::py_types::{Tensor, PY_TORCH};
+use crate::py_types::{Tensor, PY_UTILS};
 use pyo3::prelude::*;
 
 /// We need this to redirect Python stdout to Rust stdout
@@ -47,7 +47,8 @@ pub fn python_print(obj: &PyObject) {
 
 pub fn is_close(a: Tensor, b: Tensor) -> Result<PyObject, PyErr> {
     Python::with_gil(|py| {
-        PY_TORCH
+        PY_UTILS
+            .torch
             .getattr(py, "testing")
             .unwrap()
             .getattr(py, "assert_close")
@@ -78,8 +79,7 @@ pub fn opt_eval_each_subcircuit_until_fail(circuit: CircuitRc, settings: Optimiz
         if circ.info().numel() < max_numel {
             let reference = evaluate_fn(&circ).unwrap();
             let optimized = optimize_and_evaluate(circ.clone(), settings);
-            if optimized.is_ok() {
-                let optimized = optimized.unwrap();
+            if let Ok(optimized) = optimized {
                 let close_result = is_close(reference, optimized);
                 if close_result.is_err() {
                     println!("{:?}", close_result);
