@@ -1,9 +1,10 @@
 use crate::hashmaps::FxHashMap as HashMap;
+use crate::lazy::GILLazyPy;
 use crate::pyo3_prelude::*;
 use std::iter::zip;
 
 use super::{prelude::*, CachedCircuitInfo, HashBytes};
-use crate::py_types::{GENERALFUNCTIONS, PY_NONE};
+use crate::py_types::PY_UTILS;
 use crate::{
     circuit::PyCircuitBase,
     circuit_node_auto_impl, circuit_node_extra_impl, new_rc_unwrap,
@@ -11,7 +12,6 @@ use crate::{
     tensor_util::{Shape, TorchDeviceDtype},
 };
 
-use lazy_static::lazy_static;
 use macro_rules_attribute::apply;
 use pyo3::types::PyDict;
 use pyo3::types::PyTuple;
@@ -33,10 +33,10 @@ impl PyGFSpecShapeGetter {
             // println!("{}", args.getattr("__repr__").unwrap().call0().unwrap());
             let shapes: Vec<Vec<usize>> = args.extract().unwrap();
             if shapes.len() != 1 {
-                return PY_NONE.clone();
+                return PY_UTILS.none.clone();
             }
             if shapes[0].len() < self.num_non_batchable {
-                return PY_NONE.clone();
+                return PY_UTILS.none.clone();
             }
             return PyTuple::new(py, shapes[0].clone()).into();
         })
@@ -53,7 +53,7 @@ macro_rules! gfspecs_wrap {
         (
             $name.to_owned(),
             GeneralFunctionSpec {
-                function: GENERALFUNCTIONS[$name].clone(),
+                function: PY_UTILS.generalfunctions[$name].clone(),
                 get_shape: PyGFSpecShapeGetter {
                     num_non_batchable: $non_batchable,
                 }
@@ -68,25 +68,23 @@ macro_rules! gfspecs_wrap {
     };
 }
 
-lazy_static! {
-    static ref GENERAL_FUNCTIONS_SPECS: HashMap<String, GeneralFunctionSpec> =
-        Python::with_gil(|py| {
-            HashMap::from([
-                gfspecs_wrap!(py, "sigmoid", 0),
-                gfspecs_wrap!(py, "tanh", 0),
-                gfspecs_wrap!(py, "rsqrt", 0),
-                gfspecs_wrap!(py, "gelu", 0),
-                gfspecs_wrap!(py, "relu", 0),
-                gfspecs_wrap!(py, "reciprocal", 0),
-                gfspecs_wrap!(py, "log_exp_p_1", 0),
-                gfspecs_wrap!(py, "gaussian_pdf", 0),
-                gfspecs_wrap!(py, "gaussian_cdf", 0),
-                gfspecs_wrap!(py, "softmax", 1),
-                gfspecs_wrap!(py, "log_softmax", 1),
-                gfspecs_wrap!(py, "q_from_qr", 2),
-            ])
-        });
-}
+static GENERAL_FUNCTIONS_SPECS: GILLazyPy<HashMap<String, GeneralFunctionSpec>> =
+    GILLazyPy::new_py(|py| {
+        HashMap::from([
+            gfspecs_wrap!(py, "sigmoid", 0),
+            gfspecs_wrap!(py, "tanh", 0),
+            gfspecs_wrap!(py, "rsqrt", 0),
+            gfspecs_wrap!(py, "gelu", 0),
+            gfspecs_wrap!(py, "relu", 0),
+            gfspecs_wrap!(py, "reciprocal", 0),
+            gfspecs_wrap!(py, "log_exp_p_1", 0),
+            gfspecs_wrap!(py, "gaussian_pdf", 0),
+            gfspecs_wrap!(py, "gaussian_cdf", 0),
+            gfspecs_wrap!(py, "softmax", 1),
+            gfspecs_wrap!(py, "log_softmax", 1),
+            gfspecs_wrap!(py, "q_from_qr", 2),
+        ])
+    });
 /// GeneralFunctionSpec contains all needed info about function, and is the same on all instances with the same function
 /// how batchability works: input_batchability is a mask indicating which inputs support batching. if none do, there is no batching.
 /// the number of non batchable dims in output, starting from end, is num_non_batchable_output_dims.
