@@ -1,9 +1,12 @@
-use crate::hashmaps::{AHashSet as HashSet, FxHashMap as HashMap};
+use crate::{
+    hashmaps::{AHashSet as HashSet, FxHashMap as HashMap},
+    py_types::{scalar_to_tensor, tensor_scale, un_flat_concat},
+};
 use std::fmt::{self, Display};
 
 use crate::{
     cached_lambda,
-    py_types::{ExtraPySelfOps, Tensor, PY_SCALAR_TO_TENSOR, TENSOR_SCALE, UN_FLAT_CONCAT},
+    py_types::{ExtraPySelfOps, Tensor},
     pycall,
     tensor_util::{Shape, TorchDeviceDtype},
     timed,
@@ -110,7 +113,7 @@ impl Schedule {
     }
 
     pub fn split(&self, tensor: Tensor) -> Vec<Tensor> {
-        pycall!(UN_FLAT_CONCAT, (tensor, self.split_shapes.clone().unwrap()))
+        un_flat_concat(&tensor, self.split_shapes.clone().unwrap()).unwrap()
     }
 
     pub fn get_stats(&self) -> ScheduleStats {
@@ -292,17 +295,9 @@ pub fn evaluate_schedule_adjust_numerical_scale(
         })
     };
     let clamp = |tup: &(Tensor, f64)| -> (Tensor, f64) {
-        let tensor_scale: f64 = Python::with_gil(|py| {
-            TENSOR_SCALE
-                .call(py, (tup.0.clone(),), None)
-                .unwrap()
-                .extract(py)
-                .unwrap()
-        });
-        if tensor_scale > settings.numerical_scale_max
-            || tensor_scale < settings.numerical_scale_min
-        {
-            mul(tup, 1.0 / tensor_scale)
+        let scale = tensor_scale(&tup.0).unwrap();
+        if scale > settings.numerical_scale_max || scale < settings.numerical_scale_min {
+            mul(tup, 1.0 / scale)
         } else {
             tup.clone()
         }
@@ -332,26 +327,22 @@ pub fn evaluate_schedule_adjust_numerical_scale(
                 || value_scale < settings.numerical_scale_min && value_scale != 0.0
             {
                 (
-                    pycall!(
-                        PY_SCALAR_TO_TENSOR,
-                        (
-                            s.value.signum(),
-                            s.info().shape.clone(),
-                            schedule.device_dtype.clone(),
-                        )
-                    ),
+                    scalar_to_tensor(
+                        s.value.signum(),
+                        s.info().shape.clone(),
+                        schedule.device_dtype.clone(),
+                    )
+                    .unwrap(),
                     1.0 / value_scale,
                 )
             } else {
                 (
-                    pycall!(
-                        PY_SCALAR_TO_TENSOR,
-                        (
-                            s.value,
-                            s.info().shape.clone(),
-                            schedule.device_dtype.clone(),
-                        )
-                    ),
+                    scalar_to_tensor(
+                        s.value,
+                        s.info().shape.clone(),
+                        schedule.device_dtype.clone(),
+                    )
+                    .unwrap(),
                     1.0,
                 )
             }
