@@ -1,5 +1,7 @@
 #![feature(core_intrinsics)]
 #![feature(map_first_last)]
+// Personally, I like or_fun_call as a lint. But currently code fails...
+#![allow(clippy::too_many_arguments, clippy::or_fun_call)]
 
 /// reexport
 pub use pyo3;
@@ -36,6 +38,7 @@ pub mod all_imports;
 mod caching;
 pub mod circuit;
 pub mod hashmaps;
+pub mod lazy;
 pub mod opt_einsum;
 pub mod py_types;
 pub mod rearrange_spec;
