@@ -2,8 +2,8 @@ use crate::circuit::EinsumAxes;
 use crate::hashmaps::{AHashSet as HashSet, FxHashMap as HashMap};
 use crate::pyo3_prelude::*;
 use crate::{
+    circuit::py_circuit_items::PY_CIRCUIT_ITEMS,
     circuit::HashBytes,
-    py_types::PY_CIRC_COMPILER_UTIL,
     smallvec::Sv,
     sv,
     tensor_util::{Shape, TensorInvariantError},
@@ -177,7 +177,8 @@ impl RearrangeSpec {
     }
     pub fn to_py_rearrange_spec(&self, input_shape: Shape) -> PyObject {
         Python::with_gil(|py| {
-            PY_CIRC_COMPILER_UTIL
+            PY_CIRCUIT_ITEMS
+                .circ_compiler_util
                 .getattr(py, "RearrangeSpec")
                 .unwrap()
                 .getattr(py, "from_rust")
