@@ -1,9 +1,10 @@
 use crate::hashmaps::FxHashMap as HashMap;
+use crate::py_types::PY_UTILS;
 use crate::smallvec::Sv;
 use crate::{
     circuit::{CircuitConstructionError, HashBytes},
     filter_by_variant,
-    py_types::{Tensor, CAST_TENSOR, PY_CAST_INT, PY_TORCH},
+    py_types::Tensor,
     pycall,
     rearrange_spec::RearrangeSpec,
 };
@@ -190,7 +191,7 @@ impl USlice {
 
     pub fn shrink_base(&self, new_base: &USlice) -> USlice {
         assert!(new_base.stop >= self.stop);
-        assert!(self.start >= new_base.start && new_base.stop >= new_base.stop);
+        assert!(self.start >= new_base.start && self.stop >= new_base.stop);
         USlice {
             start: self.start - new_base.start,
             stop: self.stop - new_base.start,
@@ -319,19 +320,21 @@ impl TensorAxisIndex {
         seed: usize,
     ) -> Self {
         TensorAxisIndex::Tensor(pyo3::Python::with_gil(|py| {
-            PY_TORCH
+            PY_UTILS
+                .torch
                 .getattr(py, "manual_seed")
                 .unwrap()
                 .call(py, (seed,), None)
                 .unwrap();
             let mut kwargs = HashMap::new();
             if let Some(dtype) = device_dtype.dtype {
-                kwargs.insert("dtype", PY_TORCH.getattr(py, dtype).unwrap());
+                kwargs.insert("dtype", PY_UTILS.torch.getattr(py, dtype).unwrap());
             }
             if let Some(device) = device_dtype.device {
                 kwargs.insert("device", device.into_py(py));
             }
-            PY_TORCH
+            PY_UTILS
+                .torch
                 .getattr(py, "randint")
                 .unwrap()
                 .call(
@@ -515,7 +518,7 @@ pub fn compose(top: &TensorIndex, bottom: &TensorIndex) -> TensorIndex {
                     let indexed_tensor = t.clone().py_getitem(py, top_here).unwrap();
                     if indexed_tensor.shape().is_empty() {
                         result.push(TensorAxisIndex::Single(pycall!(
-                            PY_CAST_INT,
+                            PY_UTILS.cast_int,
                             (indexed_tensor,)
                         )));
                     } else {
@@ -603,7 +606,8 @@ impl TorchDeviceDtype {
 
     pub fn cast_tensor(&self, tensor: Tensor) -> Tensor {
         Python::with_gil(|py| {
-            CAST_TENSOR
+            PY_UTILS
+                .cast_tensor
                 .call(py, (tensor, self.clone()), None)
                 .unwrap()
                 .extract(py)
